/***************************************************************************//**
 *   @file   Main.c
 *   @brief  Implementation of the program's main function.
 *   @author Bancisor Mihai
********************************************************************************
 * Copyright 2012(c) Analog Devices, Inc.
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *  - Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  - Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  - Neither the name of Analog Devices, Inc. nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *  - The use of this software may or may not infringe the patent rights
 *    of one or more patent holders.  This license does not release you
 *    from the requirement that you obtain separate licenses from these
 *    patent holders to use this software.
 *  - Use of the software either in source or binary form, must be run
 *    on or directly connected to an Analog Devices Inc. component.
 *
 * THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT,
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, INTELLECTUAL PROPERTY RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
********************************************************************************
 *   SVN Revision: 755
*******************************************************************************/

/******************************************************************************/
/***************************** Include Files **********************************/
/******************************************************************************/
#include "Communication.h"
#include "ADXL362.h"
#include "Console.h"
#include "Delays.h"
/******************************************************************************/
/*********************** Device configuration words ***************************/
/******************************************************************************/
_FOSCSEL(FNOSC_PRIPLL);
_FOSC(FCKSM_CSECMD & OSCIOFNC_OFF  & POSCMD_XT);    /*!< Clock Switching Enabled and Fail Safe Clock Monitor is disabled*/
                                                    /*!< OSC2 Pin Function: OSC2 is Clock Output*/
                                                    /*!  Primary Oscillator Mode: XT Crystal*/
_FWDT(FWDTEN_OFF); 			            /*!< Watchdog Timer Enabled/disabled by user software*/

/******************************************************************************/
/************************ Constants Definitions *******************************/
/******************************************************************************/
/* List of available commands */
const char* commandsList[] = {"help?",
                              "id?",
                              "measure=",
                              "temp?",
                              "reset=",
                              "acceleration?",
                              "activity?"};
const char* commandsDescription[] = {
    "  -  Displays all available commands.",
    "  -  Device details.",
    "  -  Start/stop the measure process of the device. Accepted values: 0 - 1.",
    "  -  Read the temperature.",
    "  -  Reset the device.",
    "  -  Displays the accelerations on XYZ axis.",
    "  -  Displays the activity status of the device. It runs for 5 motion detections."};
/******************************************************************************/
/************************ Variables Definitions *******************************/
/******************************************************************************/
unsigned char commandsNumber = (sizeof(commandsList) / sizeof(const char*));
unsigned char receivedCommand[20];
unsigned char invalidCommand = 0;
unsigned char commandType = 0;
unsigned char command = 0;
unsigned char displayCommand = 0;
double        commandParam = 0;

/***************************************************************************//**
 * @brief Converts a float value to a character array with 3 digits of accuracy.
 *
 * @param *buf - returns the converterd value
 * @param val - value to be converted
 *
 * @return None.
*******************************************************************************/
void FloatToString(char * buf, double val)
{
    long  intPart  = 0;
    short fracPart = 0;
    short  charPos  = 0;
    char  localBuf[10] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
    short  i = sizeof(localBuf) - 1;

    intPart = (long)val;
    fracPart = (short)((val - intPart) * 1000 + 0.5);
    while(i > sizeof(localBuf) - 4)
    {
        localBuf[i] = (fracPart % 10) + 0x30;
        fracPart /= 10;
        i--;
    }
    localBuf[i] = '.';
    if(intPart == 0)
    {
        i --;
        localBuf[i] = '0';
    }
    while(intPart)
    {
        i --;
        localBuf[i] =(intPart % 10)  + 0x30;
        intPart /= 10;
    }
    for(charPos = i; charPos < sizeof(localBuf); charPos ++)
    {
        *buf = localBuf[charPos];
        buf ++;
    }
	*buf = 0;
}
/***************************************************************************//**
 * @brief Main function.
 *
 * @return None.
*******************************************************************************/
int main(void)
{
    /*! Configure PLL prescaler, PLL postscaler, PLL divisor. */
    /*! Fosc = 80MHz, Fcy = 40MHz, 40MIPS. */
    PLLFBD = 38;            // M = PLLFBD + 2
    CLKDIVbits.PLLPOST = 0; // N2 = 2
    CLKDIVbits.PLLPRE = 0;  // N1 = 2

    /* Variables holding information about the device */
    short          xAxisAcc = 0;
    short          yAxisAcc = 0;
    short          zAxisAcc = 0;
    unsigned char  regVal = 0;
    float          degCelsius = 0;

    /* Temporary variables */
    char  tempString[10] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
    unsigned long  samplesNumber  = 0;
    unsigned char  detections     = 0;
    long           intCommand     = 0;

    /*! Wait for PLL to lock. */
    while(OSCCONbits.COSC != 0b011);
    while(OSCCONbits.LOCK != 1);

    T1CONbits.TON = 0;      // Disable Timer.
    T1CONbits.TCS = 0;      // Select internal instruction cycle clock.
    T1CONbits.TGATE = 0;    // Disable Gated Timer mode.
    T1CONbits.TCKPS = 0b01; // Select 1:1 Prescaler.
                            // 1 tick ~ 0.2 us.
    TMR1 = 0x00;            // Clear timer register.
    PR1 = 0xFFFF;           // Load the period value.
    T1CONbits.TON = 1;      // Start Timer.

    TRISD |= (1 << 10);
    TRISD |= (1 << 11);

    /*! Initialize the UART communication peripheral. */
    UART_Init(9600);
    /*! Initialize the AD5160 device. */
    /* Initialize the ADXL362 device. */
    if(ADXL362_Init())
    {
        CONSOLE_WriteString("ADXL362 OK");
        UART_Write(0x0D);
    }
    else
    {
        CONSOLE_WriteString("ADXL362 Error");
        UART_Write(0x0D);
    }
    while(1)
    {
        CONSOLE_GetCommand(receivedCommand);
        invalidCommand = 0;
        for(command = 0; command < commandsNumber; command++)
        {
            commandType = CONSOLE_CheckCommands(receivedCommand,
                                                commandsList[command],
                                                (double*)&commandParam);
            if(commandType == 0)
            {
                invalidCommand++;
            }
            if((command == 0) && (commandType != 0))    // "help?" command
            {
                CONSOLE_WriteString("Available commands:");
                UART_Write(0x0D);
                for(displayCommand = 0; displayCommand < commandsNumber;
                    displayCommand++)
                {
                    CONSOLE_WriteString(commandsList[displayCommand]);
                    CONSOLE_WriteString(commandsDescription[displayCommand]);
                    UART_Write(0x0D);
                }
            }
            if((command == 1) && (commandType != 0))	// "id?" command
            {
                /* Send the requested value to user. */
                CONSOLE_WriteString("Analog Devices device ID: ");
                ADXL362_GetRegisterValue(&regVal, ADXL362_REG_DEVID_AD, 1);
                itoa(tempString, regVal, 16);
                CONSOLE_WriteString(tempString);
                UART_Write(0xD);
                CONSOLE_WriteString("Analog Devices MEMS ID: ");
                ADXL362_GetRegisterValue(&regVal, ADXL362_REG_DEVID_MST, 1);
                itoa(tempString, regVal, 16);
                CONSOLE_WriteString(tempString);
                UART_Write(0xD);
                CONSOLE_WriteString("Analog Devices PART ID: ");
                ADXL362_GetRegisterValue(&regVal, ADXL362_REG_PARTID, 1);
                itoa(tempString, regVal, 8);
                CONSOLE_WriteString(tempString);
                UART_Write(0xD);
                CONSOLE_WriteString("Analog Devices REVISION ID: ");
                ADXL362_GetRegisterValue(&regVal, ADXL362_REG_REVID, 1);
                itoa(tempString, regVal, 10);
                CONSOLE_WriteString(tempString);
                UART_Write(0xD);
            }
            if((command == 2) && (commandType != 0))    // "measure=" command
            {
                /* Turn on/off the measurement process of the device. */
                intCommand = (long)(commandParam);
                if((intCommand == 1) || (intCommand == 0))
                {
                    ADXL362_SetPowerMode(intCommand);
                }
                else
                {
                    intCommand = 0;
                }
                /* Send feedback to user. */
                CONSOLE_WriteString(commandsList[command]);
                itoa(tempString, intCommand, 10);
                CONSOLE_WriteString(tempString);
                UART_Write(0xD);
            }
            if((command == 3) && (commandType != 0))    // "temperature?" command
            {
                /* Read the temperature of the device. */
                degCelsius = ADXL362_ReadTemperature();
                FloatToString(tempString, degCelsius);
                CONSOLE_WriteString(tempString);
                UART_Write(0xD);
            }
            if((command == 4) && (commandType != 0))    // "reset=" command
            {
                /* Resets the device(software reset). */
                intCommand = (long)commandParam;
                if(intCommand == 1)
                {
                    ADXL362_SoftwareReset();
                }
                else
                {
                    intCommand = 0;
                }
                /* Send feedback to user */
                CONSOLE_WriteString(commandsList[command]);
                itoa(tempString, intCommand, 10);
                CONSOLE_WriteString(tempString);
                UART_Write(0xD);
            }
            if((command == 5) && (commandType != 0))    // "acceleration?" command
            {
                /* Read and displays 30 samples from the accelerometer. */
                samplesNumber = 30;
                while(samplesNumber > 0)
                {
                    /* Read the requested value from the device. */
                    ADXL362_GetXyz(&xAxisAcc, &yAxisAcc, &zAxisAcc);
                    /* Send the requested value to user. */
                    CONSOLE_WriteString("x axis: ");
                    itoa(tempString, xAxisAcc, 10);
                    CONSOLE_WriteString(tempString);
                    UART_Write(0xD);
                    CONSOLE_WriteString("y axis: ");
                    itoa(tempString, yAxisAcc, 10);
                    CONSOLE_WriteString(tempString);
                    UART_Write(0xD);
                    CONSOLE_WriteString("z axis: ");
                    itoa(tempString, zAxisAcc, 10);
                    CONSOLE_WriteString(tempString);
                    UART_Write(0xD);
                    DelayMs(100);
                    samplesNumber--;
                }
            }
            if((command == 6) && (commandType != 0))    // "activity?" command
            {
                /* Setup the activity and inactivity detection. */
                CONSOLE_WriteString("Activity polling enabled.");
                UART_Write(0x0D);
                ADXL362_SetPowerMode(0);
                ADXL362_SetOutputRate(ADXL362_ODR_100_HZ);
                ADXL362_SetRegisterValue(
                              ADXL362_ACT_INACT_CTL_LINKLOOP(ADXL362_MODE_LINK),
                              ADXL362_REG_ACT_INACT_CTL,
                              1);
                ADXL362_SetupActivityDetection(1, 60, 4);
                ADXL362_SetupInactivityDetection(1, 700, 250);
                ADXL362_SetPowerMode(1);
                /* Clear ACT and INACT bits by reading the Status Register. */
                ADXL362_GetRegisterValue(&regVal,
                                         ADXL362_REG_STATUS,
                                         1);
                /* Exit polling after 5 detections. */
                detections = 5;
                while(detections)
                {
                    do // Wait for the detection of an activity or inactivity.
                    {
                        ADXL362_GetRegisterValue(&regVal,
                                                 ADXL362_REG_STATUS,
                                                 1);
                    }while(!(regVal & ADXL362_STATUS_ACT) &&
                           !(regVal & ADXL362_STATUS_INACT));
                    detections--;
                    if(regVal & ADXL362_STATUS_ACT)
                    {
                        CONSOLE_WriteString("Activity");
                        UART_Write(0x0D);
                    }
                    if(regVal & ADXL362_STATUS_INACT)
                    {
                        CONSOLE_WriteString("Inactivity");
                        UART_Write(0x0D);
                    }
                }
                CONSOLE_WriteString("Finished activity polling.");
                UART_Write(0x0D);
            }
        }
        if(invalidCommand == commandsNumber)
        {
            /* Send feedback to user. */
            CONSOLE_WriteString("Invalid command");
            UART_Write(0x0D);
        }
    }
}